import { Cleanable, TeardownLogic } from '../types';
import { Data, Computation } from './internal';
import { RunnableGenerator } from '../concurrency/types';
export declare class Cell<T> {
    protected internal: Data<T>;
    constructor(internal: Data<T>);
    get value(): T;
    [Symbol.iterator](): RunnableGenerator<T>;
    map<R>(mapFn: (value: T) => R): Cell<R>;
    watch(watchFn: (value: T) => Cleanable, options?: WatchOptions): TeardownLogic;
    /**
     * @deprecated Use watch.
     */
    subscribe(next: (value: T) => void): {
        (): void;
        unsubscribe(): void;
    };
}
export interface WatchOptions {
    immediate?: boolean;
}
export declare class ComputationalCell<T> extends Cell<T> {
    constructor(internal: Computation<T>);
    get value(): T;
}
export declare class Lazy<T> {
    private internal;
    get stale(): boolean;
    constructor(internal: Computation<T>);
    execute(expr: (current: T) => T): T;
    watch(watchFn: (value: T) => Cleanable, options?: WatchOptions): TeardownLogic;
}
export declare function mutable<T>(initialValue: T): [
    Cell<T>,
    (value: (T extends Function ? never : T) | ((current: T) => T)) => void
];
export declare function mutValue<T>(initialValue: T): [Cell<T>, (value: T) => void];
export declare function lazy<T>(initial?: T): Lazy<T>;
export declare function computed<T>(expr: () => T, options?: {
    static?: boolean;
    initial?: T;
}): Cell<T>;
export declare function suspended<T, F = T>(expr: () => T, fallback: F): Cell<T>;
export declare function constant<T>(value: T): Cell<T>;
export declare type UnwrapProperty<T> = T extends object ? {
    [P in keyof T]: T[P] extends Cell<infer C> ? C : T[P];
} : T;
export declare function combined<A extends Array<Cell<any>>, R = UnwrapProperty<A>>(array: A, lifeFn?: (a: UnwrapProperty<A>) => R): Cell<R>;
export declare function combined<C extends {
    [key: string]: Cell<any>;
}, R = UnwrapProperty<C>>(obj: C, lifeFn?: (A: UnwrapProperty<C>) => R): Cell<R>;
