declare const enum Flag {
    /**
     * this is a data */
    Data = 1,
    /**
     * this is a computation */
    Computation = 2,
    /**
     * (computation) current value is stale
     */
    Stale = 4,
    /**
     * current value is changed (so need to propagate)
     */
    Changed = 8,
    /**
     * (computation) current value maybe stale
     */
    MarkForCheck = 32,
    /**
     *
     */
    Estimating = 16,
    /**
     * dynamic is true when this is true
     */
    Suspensed = 64,
    Suspending = 128,
    Lazy = 256,
    Propagating = 512
}
declare class Suspend {
    readonly cancel: () => void;
    constructor(cancel: () => void);
}
declare class SuspendWithFallback extends Suspend {
    readonly fallback: any;
    readonly cancel: () => void;
    constructor(fallback: any, cancel: () => void);
}
interface WatcherLinkedNode {
    fn: Function;
    prev: WatcherLinkedNode | null;
    next: WatcherLinkedNode | null;
    disposed: boolean;
    data: Data;
}
interface SourceLinkedNode<T> {
    prev_source: SourceLinkedNode<T> | null;
    next_source: SourceLinkedNode<T> | null;
    source: T;
    observer_ref: ObserverLinkedNode<any>;
}
interface ObserverLinkedNode<T> {
    prev_observer: ObserverLinkedNode<T> | null;
    next_observer: ObserverLinkedNode<T> | null;
    observer: T;
}
interface Data<T = any> {
    flags: Flag;
    value: T | undefined;
    last_effect: WatcherLinkedNode | null;
    last_observer: ObserverLinkedNode<Computation> | null;
    depsCounter: number;
}
interface Computation<T = any> extends Data<T> {
    checkNode: SourceLinkedNode<Data> | null;
    cleanNode: SourceLinkedNode<Data> | null;
    first_source: SourceLinkedNode<Data> | null;
    last_source: SourceLinkedNode<Data> | null;
    collect: Function;
}
interface Suspense<T = any, F = undefined> extends Computation<T> {
    fallback: F;
    currentCancel: (() => void) | undefined;
}
declare function setData<T>(data: Data<T>, value: T, equalCheck: boolean): void;
declare function accessData(data: Data): any;
declare function accessComputation<T>(data: Computation<T>): T | undefined;
declare function untrack<T>(fn: (...args: any[]) => T, ...args: any[]): T;
declare function watch(data: Data, sideEffect: Function): WatcherLinkedNode;
declare function disposeWatcher(watcher: WatcherLinkedNode): void;
declare function transaction<T>(fn: () => T): T;
declare function createLazy<T>(initial?: T): Computation<T>;
declare function executeLazy<T>(computation: Computation<T>, fn: (current: T) => T): T;
declare function createData<T>(value: T): Data<T>;
declare function createComputation<T>(fn: (current: T) => T, options?: {
    initial?: T;
    static?: boolean;
    sources?: Data[];
}): Computation<T>;
declare function createSuspended<T, F>(fn: (current: T | F) => T, fallback: F): Suspense<T, F>;
export declare function __current_collecting(): Computation<any> | null;
export { Computation, Data, Flag, transaction, setData, watch, disposeWatcher, accessData, accessComputation, createData, createComputation, untrack, createLazy, executeLazy, createSuspended, Suspend, SuspendWithFallback, };
