import { Action, Subscribable, TeardownLogic } from '../types';
import { CancellablePromise } from './promise';
import { Runnable } from './types';
export declare function executeRunnable<T>(runnable: Runnable<T>, onsuccess: Action<T>, onfailure: Action<any>): TeardownLogic;
declare type GeneratorReturnType<T> = T extends () => Runnable<infer R> ? R : undefined;
export interface Task<T> extends PromiseLike<T>, Runnable<T> {
    cancel(): void;
}
export declare function task<TaskFn extends (...args: any[]) => Runnable<any>>(taskFn: TaskFn): (...params: Parameters<TaskFn>) => Task<GeneratorReturnType<TaskFn>>;
export declare function start<T>(runnable: Runnable<T>): CancellablePromise<T>;
export declare function delay(time: number): Runnable<void>;
export declare function timeout(time: number): Runnable<void>;
export declare function nextAnimationFrame(): Runnable<number>;
export declare class Semaphore {
    readonly maxConcurrency: number;
    private currentNum;
    private queue;
    constructor(maxConcurrency: number);
    get free(): boolean;
    waitOne(): Runnable<void>;
    release(): void;
}
declare function resolve<T>(thenable: PromiseLike<T>): Runnable<T>;
declare function resolve<T>(observable: Subscribable<T>): Runnable<T>;
declare function resolve<T>(runnable: Runnable<T>): Runnable<T>;
declare function resolve<T extends object>(value: T): Runnable<never>;
declare function resolve<T>(value: T): Runnable<T>;
declare type Resolve<T> = T extends PromiseLike<infer P> ? P : T extends Subscribable<infer S> ? S : T extends Runnable<infer R> ? R : T;
declare type ResolveAll<T> = {
    [P in keyof T]: Resolve<T[P]>;
};
declare function any<T>(tasks: Iterable<T>): Runnable<ResolveAll<T>>;
declare function race<T>(tasks: Iterable<T>): Runnable<ResolveAll<T>>;
declare function all<T>(tasks: Iterable<T>): Runnable<Resolve<T>[]>;
declare type TaskSettlement<T> = {
    success: true;
    value: T;
} | {
    success: false;
    value: any;
};
declare function allSettled<T>(tasks: Iterable<T>): Runnable<TaskSettlement<Resolve<T>>[]>;
export { resolve, all, allSettled, any, race };
